@@ -69,64 +69,6 @@ enum_with_str_repr! {
     }
 }
 
-enum_with_str_repr! {
-    Intensity {
-        Light => "-",
-        Moderate => "",
-        Heavy => "+",
-    }
-}
-
-enum_with_str_repr! {
-    Descriptor {
-        Shallow => "MI",
-        Partial => "PR",
-        Patches => "BC",
-        LowDrifting => "DR",
-        Blowing => "BL",
-        Showers => "SH",
-        Thunderstorm => "TS",
-        Freezing => "FZ",
-    }
-}
-
-enum_with_str_repr! {
-    Precipitation {
-        Rain => "RA",
-        Drizzle => "DZ",
-        Snow => "SN",
-        SnowGrains => "SG",
-        IceCrystals => "IC",
-        IcePellets => "PL",
-        Hail => "GR",
-        Graupel => "GS",
-        Unknown => "UP",
-    }
-}
-
-enum_with_str_repr! {
-    Obscuration {
-        Fog => "FG",
-        Mist => "BR",
-        Haze => "HZ",
-        VolcanicAsh => "VA",
-        WidespreadDust => "DU",
-        Smoke => "FU",
-        Sand => "SA",
-        Spray => "PY",
-    }
-}
-
-enum_with_str_repr! {
-    Other {
-        Squall => "SQ",
-        SandWhirls => "PO",
-        Duststorm => "DS",
-        Sandstorm => "SS",
-        FunnelCloud => "FC",
-    }
-}
-
 enum_with_str_repr! {
     CloudType {
         Cumulonimbus => "CB",
@@ -141,45 +83,58 @@ enum_with_str_repr! {
 #[derive(Clone, Copy, PartialEq, Eq, Debug)]
 pub struct ZuluDateTime {
     pub day_of_month: u8,
-    pub hour: u8,
-    pub minute: u8,
+    pub time: ZuluTime,
     /// Some stations omit the Z. It is unclear whether this means
-    /// that the timestamp is not zulu, or if it is incorrect implementation.
+    /// that the timestamp is not zulu, or if it is incorrect implementation
     pub is_zulu: bool,
 }
 
 impl ZuluDateTime {
     #[cfg(feature = "chrono_helpers")]
     pub fn as_datetime(&self, year: i32, month: u32) -> chrono::DateTime<chrono_tz::Tz> {
-        chrono::TimeZone::ymd(&chrono_tz::Greenwich, year, month, self.day_of_month as u32).and_hms(
-            self.hour as u32,
-            self.minute as u32,
-            0,
-        )
+        self.time.as_datetime(chrono::TimeZone::ymd(
+            &chrono_tz::Greenwich,
+            year,
+            month,
+            self.day_of_month as u32,
+        ))
+    }
+}
+
+#[derive(Clone, Copy, PartialEq, Eq, Debug)]
+pub struct ZuluTime {
+    pub hour: u8,
+    pub minute: u8,
+}
+
+impl ZuluTime {
+    #[cfg(feature = "chrono_helpers")]
+    pub fn as_datetime(
+        &self,
+        date: chrono::Date<chrono_tz::Tz>,
+    ) -> chrono::DateTime<chrono_tz::Tz> {
+        date.and_hms(self.hour as u32, self.minute as u32, 0)
     }
 }
 
 #[derive(Clone, Copy, PartialEq, Eq, Debug)]
 pub struct ZuluTimeRange {
-    pub begin_hour: u8,
-    pub begin_minute: u8,
-    pub end_hour: u8,
-    pub end_minute: u8,
+    pub begin: ZuluTime,
+    pub end: ZuluTime,
 }
 
 impl ZuluTimeRange {
     #[cfg(feature = "chrono_helpers")]
-    pub fn as_datetime_range(
+    pub fn as_start_and_duration(
         &self,
         date: chrono::Date<chrono_tz::Tz>,
     ) -> (
         chrono::DateTime<chrono_tz::Tz>,
-        chrono::DateTime<chrono_tz::Tz>,
+        chrono::Duration,
     ) {
-        (
-            date.and_hms(self.begin_hour as u32, self.begin_minute as u32, 0),
-            date.and_hms(self.end_hour as u32, self.end_minute as u32, 0),
-        )
+        let begin = self.begin.as_datetime(date);
+        let end = self.end.as_datetime(date);
+        (begin, (end - begin))
     }
 }
 
@@ -220,8 +175,9 @@ pub enum VisibilityType {
 
 #[derive(Clone, Copy, PartialEq, Debug)]
 pub struct RawRunwayVisibility {
-    pub value: Length,
+    /// If present, visibility is out of the observable range
     pub out_of_range: Option<OutOfRange>,
+    pub value: Length,
 }
 
 enum_with_str_repr! {
@@ -247,29 +203,90 @@ pub struct RunwayReport<'input> {
 
 #[derive(Clone, Copy, PartialEq, Debug)]
 pub enum RunwayReportInfo {
+    /// Runway has been cleared of any deposits
     Cleared { friction: Option<f64> },
 }
 
 #[derive(Clone, PartialEq, Eq, Debug)]
 pub struct Weather {
     pub intensity: Intensity,
+    /// If true, the described phenomenon was not observed above the station, but within 8KM of it
     pub vicinity: bool,
     pub descriptor: Option<Descriptor>,
     /// Condition should always be present, unless this is a VCTS or VCSH
     pub condition: Option<Condition>,
 }
 
+enum_with_str_repr! {
+    Intensity {
+        Light => "-",
+        Moderate => "",
+        Heavy => "+",
+    }
+}
+
+enum_with_str_repr! {
+    Descriptor {
+        Shallow => "MI",
+        Partial => "PR",
+        Patches => "BC",
+        LowDrifting => "DR",
+        Blowing => "BL",
+        Showers => "SH",
+        Thunderstorm => "TS",
+        Freezing => "FZ",
+    }
+}
+
 #[derive(Clone, PartialEq, Eq, Debug)]
 pub enum Condition {
+    /// There can be multiple forms of precipitation observed
     Precipitation(Vec<Precipitation>),
     Obscuration(Obscuration),
     Other(Other),
 }
 
+enum_with_str_repr! {
+    Precipitation {
+        Rain => "RA",
+        Drizzle => "DZ",
+        Snow => "SN",
+        SnowGrains => "SG",
+        IceCrystals => "IC",
+        IcePellets => "PL",
+        Hail => "GR",
+        Graupel => "GS",
+        Unknown => "UP",
+    }
+}
+
+enum_with_str_repr! {
+    Obscuration {
+        Fog => "FG",
+        Mist => "BR",
+        Haze => "HZ",
+        VolcanicAsh => "VA",
+        WidespreadDust => "DU",
+        Smoke => "FU",
+        Sand => "SA",
+        Spray => "PY",
+    }
+}
+
+enum_with_str_repr! {
+    Other {
+        Squall => "SQ",
+        SandWhirls => "PO",
+        Duststorm => "DS",
+        Sandstorm => "SS",
+        FunnelCloud => "FC",
+    }
+}
+
 #[derive(Clone, Copy, PartialEq, Debug)]
 pub struct CloudCover {
     pub coverage: CloudCoverage,
-    /// The absence of a base indicates it is below station level or an inability to assess
+    /// The absence of a base indicates it is below station level or an inability of an automated system to make an assessment
     pub base: Option<Length>,
     pub cloud_type: Option<CloudType>,
 }
@@ -278,15 +295,15 @@ pub struct CloudCover {
 #[derive(Clone, Copy, PartialEq, Debug)]
 pub struct Temperatures {
     pub air: ThermodynamicTemperature,
-    /// Some stations don't report this
+    /// Some stations don't report this, hence it is marked as optional
     pub dewpoint: Option<ThermodynamicTemperature>,
 }
 
 #[derive(Clone, Copy, PartialEq, Debug)]
 pub struct AccumulatedRainfall {
-    /// In the last 10 minutes
+    /// In the 10 minutes prior to the report time
     pub recent: Length,
-    /// Since 0900 (presumably local time)
+    /// Since 0900 local
     pub past: Length,
 }
 
@@ -306,10 +323,21 @@ enum_with_str_repr! {
     }
 }
 
+#[derive(Clone, Copy, PartialEq, Debug)]
+pub struct Color {
+    /// `BLACK` in a METAR indicates the airfield is closed
+    pub is_black: bool,
+    pub current_color: ColorState,
+    pub next_color: Option<ColorState>,
+}
+
 #[derive(Clone, Copy, PartialEq, Debug)]
 pub struct Visibility {
     pub prevailing: Length,
+    /// Typically reported when visibility in a particular direction differs significantly from prevailing visibility
+    /// If there is more than one direction, the most operationally significant direction is used
     pub minimum_directional: Option<DirectionalVisibility>,
+    /// Reported in addition to the minimum when there is large difference in directional visibility (i.e. 1500M vs 5000M)
     pub maximum_directional: Option<DirectionalVisibility>,
 }
 
@@ -335,6 +363,7 @@ enum_with_str_repr! {
 
 #[derive(Clone, Copy, PartialEq, Debug)]
 pub struct WaterConditions {
+    /// Water temperature at the surface
     pub temperature: Option<ThermodynamicTemperature>,
     pub surface_state: Option<WaterSurfaceState>,
     pub significant_wave_height: Option<Length>,
@@ -360,7 +389,9 @@ enum_with_str_repr! {
 pub enum Trend {
     /// No significant change in weather expected for the next 2 hours
     NoSignificantChange,
+    /// Expected changes
     Becoming(TrendReport),
+    /// Temporary fluctuations that last less than an hour
     Temporarily(TrendReport),
 }
 
@@ -377,7 +408,7 @@ pub struct TrendReport {
 #[derive(Clone, Copy, PartialEq, Eq, Debug)]
 pub struct TrendTime {
     pub time_type: TrendTimeType,
-    pub time: u16,
+    pub time: ZuluTime,
 }
 
 enum_with_str_repr! {
@@ -390,15 +421,21 @@ enum_with_str_repr! {
 
 #[derive(Clone, PartialEq, Debug)]
 pub struct MetarReport<'input> {
+    /// Station [ICAO identifier](https://en.wikipedia.org/wiki/ICAO_airport_code)
     pub identifier: &'input str,
     pub observation_time: ZuluDateTime,
+    /// Usually used by TAFs, but some stations include this
     pub observation_validity_range: Option<ZuluTimeRange>,
     pub observation_type: Option<ObservationType>,
     pub wind: Option<Wind>,
     pub visibility: Option<Visibility>,
+    /// Included by some airport stations
     pub runway_visibilities: Vec<RunwayVisibility<'input>>,
+    /// Included by some airport stations
     pub runway_reports: Vec<RunwayReport<'input>>,
+    /// Series of active weather conditions
     pub weather: Vec<Weather>,
+    /// Describes observed cloud layers at different heights
     pub cloud_cover: Vec<CloudCover>,
     /// Indicative of OK ceiling and visibility
     ///
@@ -406,18 +443,22 @@ pub struct MetarReport<'input> {
     pub cavok: bool,
     pub temperatures: Option<Temperatures>,
     pub pressure: Option<Pressure>,
+    /// Often reported by Australian stations
+    ///
+    /// See the Australian Government [Bureau of Meteorology FAQ](http://www.bom.gov.au/aviation/about-us/faq/)
     pub accumulated_rainfall: Option<AccumulatedRainfall>,
-    /// `BLACK` in a METAR report indicates the airfield is closed
-    pub is_closed: bool,
-    pub color_state: Option<ColorState>,
-    /// Some stations will report the next color state
-    pub next_color_state: Option<ColorState>,
+    /// Often reported by military stations
+    pub color: Option<Color>,
     pub recent_weather: Vec<Weather>,
+    /// Often reported by stations at sea
+    ///
+    /// i.e. [ENQA](https://en.wikipedia.org/wiki/Troll_A_platform), an offshore natural gas platform.
     pub water_conditions: Option<WaterConditions>,
     pub trends: Vec<Trend>,
+    /// Additional information outside of the METAR specification
     pub remark: Option<&'input str>,
-    /// Some automated METAR reports indicate if the system needs maintenance
+    /// Some automated METARs indicate if the system needs maintenance
     ///
-    /// This may indicate unreliable measurements.
+    /// This may indicate that measurements are unreliable
     pub maintenance_needed: bool,
 }
